@@ -1,13 +1,6 @@
-#define __GNOME_PRINT_PREVIEW_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Copyright (C) 1999-2001 Ximian Inc. and authors
- *
- *  Authors:
- *    Miguel de Icaza (miguel@gnu.org)
- *    Lauris Kaplinski <lauris@ximian.com>
- *
- *  Preview driver
+ *  gnome-print-preview.c: print preview driver
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -22,15 +15,25 @@
  *  You should have received a copy of the GNU Library General Public
  *  License along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *  Authors:
+ *    Miguel de Icaza <miguel@ximian.com>
+ *    Lauris Kaplinski <lauris@ximian.com>
+ *
+ *  Copyright (C) 1999-2002 Ximian Inc. and authors
+ *
  */
 
+#include <config.h>
+
 #include <string.h>
 #include <math.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include <libart_lgpl/art_misc.h>
-#include <libart_lgpl/art_pixbuf.h>
+#include <libgnomecanvas/gnome-canvas-clipgroup.h>
+#include <libgnomeprint/private/gp-gc-private.h>
+#include <libgnomeprint/private/gnome-glyphlist-private.h>
 
-#include <gtk/gtksignal.h>
 #include "gnome-canvas-hacktext.h"
 #include "gnome-print-preview-private.h"
 
@@ -46,9 +49,26 @@ struct _GnomePrintPreviewPrivate {
 
 	/* The current page */
 	GnomeCanvasItem *page;
+
+	/* Strict theme compliance [#96802] */
+	gboolean theme_compliance; 
 };
 
 static GnomePrintContextClass *parent_class;
+static gboolean use_theme;
+
+static void
+outline_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
+{
+	gint32 color;
+	GtkStyle *style;
+
+	style = gtk_widget_get_style (GTK_WIDGET (canvas));
+	color = GPP_COLOR_RGBA (style->text [GTK_STATE_NORMAL], 0xff);
+	
+	gnome_canvas_item_set (item, "outline_color_rgba", color, NULL);
+}
+	
 
 static int
 gpp_stroke (GnomePrintContext *pc, const ArtBpath *bpath)
@@ -74,18 +94,32 @@ gpp_stroke (GnomePrintContext *pc, const ArtBpath *bpath)
 		gnome_canvas_bpath_get_type (),
 		"bpath",	path,
 		"width_units",	gp_gc_get_linewidth (pc->gc),
-		"cap_style",	gp_gc_get_linecap (pc->gc),
+		"cap_style",	gp_gc_get_linecap (pc->gc) + 1 /* See #104932 */,
 		"join_style",	gp_gc_get_linejoin (pc->gc),
 		"outline_color_rgba", gp_gc_get_rgba (pc->gc),
 		"miterlimit",	gp_gc_get_miterlimit (pc->gc),
 		"dash",		gp_gc_get_dash (pc->gc),
 		NULL);
 
-	gp_path_unref (path);
+	gnome_canvas_path_def_unref (path);
 
+	if (use_theme)
+		outline_set_style_cb (GTK_WIDGET (item->canvas), item);	
 	return 1;
 }
 
+static void
+fill_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
+{
+	gint32 color;
+	GtkStyle *style;
+
+	style = gtk_widget_get_style (GTK_WIDGET (canvas));
+	color = GPP_COLOR_RGBA (style->bg [GTK_STATE_NORMAL], 0xff);
+	
+	gnome_canvas_item_set (item, "fill_color_rgba", color, NULL);
+}
+	
 static int
 gpp_fill (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 {
@@ -111,9 +145,10 @@ gpp_fill (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 		"fill_color_rgba", gp_gc_get_rgba (pc->gc),
 		"wind", rule,
 		NULL);
+	gnome_canvas_path_def_unref (path);
 
-	gp_path_unref (path);
-
+	if (use_theme)
+		fill_set_style_cb (GTK_WIDGET (item->canvas), item);
 	return 1;
 }
 
@@ -141,18 +176,25 @@ gpp_clip (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 
 	gp_gc_set_data (priv->gc, clip);
 
-	gp_path_unref (path);
+	gnome_canvas_path_def_unref (path);
 
 	return 1;
 }
 
+static void
+gpp_image_free_pix (guchar *pixels, gpointer data)
+{
+	g_free (pixels);
+
+}
+
 static int
 gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint w, gint h, gint rowstride, gint ch)
 {
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
 	GnomeCanvasGroup * group;
 	GnomeCanvasItem *item;
-	ArtPixBuf *pixbuf;
+	GdkPixbuf *pixbuf;
 	int size, bpp;
 	void *dup;
 	
@@ -167,15 +209,19 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 	}
 	
 	size = w * h * bpp;
-	dup = art_alloc (size);
+	dup = g_malloc (size);
 	if (!dup) return -1;
 
 	if (ch == 3) {
 		memcpy (dup, px, size);
-		pixbuf = art_pixbuf_new_rgb (dup, w, h, rowstride);
+		pixbuf = gdk_pixbuf_new_from_data (dup, GDK_COLORSPACE_RGB,
+						   FALSE, 8, w, h, rowstride,
+						   gpp_image_free_pix, NULL);
 	} else if (ch == 4) {
 		memcpy (dup, px, size);
-		pixbuf = art_pixbuf_new_rgba (dup, w, h, rowstride);
+		pixbuf = gdk_pixbuf_new_from_data (dup, GDK_COLORSPACE_RGB,
+				                   TRUE, 8, w, h, rowstride,
+						   gpp_image_free_pix, NULL);
 	} else if (ch == 1) {
 		const char *source;
 		char *target;
@@ -192,22 +238,24 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 				source++;
 			}
 		}
-		pixbuf = art_pixbuf_new_rgb (dup, w, h, rowstride * 3);
+		pixbuf = gdk_pixbuf_new_from_data (dup, GDK_COLORSPACE_RGB,
+						   FALSE, 8, w, h, rowstride * 3,
+						   gpp_image_free_pix, NULL);
 	} else
 		return -1;
 
 	group = (GnomeCanvasGroup *) gp_gc_get_data (pp->priv->gc);
 
 	item = gnome_canvas_item_new (group,
-		GNOME_TYPE_CANVAS_PIXBUF,
-		"pixbuf", pixbuf,
-		"x",      0.0,
-		"y",      0.0,
-		"width",  (gdouble) w,
-		"height", (gdouble) h,
-		"anchor", GTK_ANCHOR_NW,
-		NULL);
-
+				      GNOME_TYPE_CANVAS_PIXBUF,
+				      "pixbuf", pixbuf,
+				      "x",      0.0,
+				      "y",      0.0,
+				      "width",  (gdouble) w,
+				      "height", (gdouble) h,
+				      "anchor", GTK_ANCHOR_NW,
+				      NULL);
+	g_object_unref (G_OBJECT (pixbuf));
 
 	/* Apply the transformation for the image */
 	{
@@ -219,9 +267,7 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 		flip[3] = -1.0 / h;
 		flip[4] = 0.0;
 		flip[5] = 1.0;
-#if 0
-		art_affine_scale (flip, 1.0, -1.0);
-#endif
+
 		art_affine_multiply (transform, flip, affine);
 		gnome_canvas_item_affine_absolute (item, transform);
 	}
@@ -232,13 +278,34 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 static int
 gpp_showpage (GnomePrintContext *pc)
 {
-	return 0;
+	return GNOME_PRINT_OK;
 }
 
 static int
 gpp_beginpage (GnomePrintContext *pc, const guchar *name)
 {
-	return 0;
+	return GNOME_PRINT_OK;
+}
+
+static void
+glyphlist_set_style_cb (GtkWidget *canvas, GnomeCanvasItem *item)
+{
+	GnomeGlyphList *gl, *new;
+	gint32 color;
+	GtkStyle *style;
+	gint i;
+
+	style = gtk_widget_get_style (GTK_WIDGET (canvas));
+	color = GPP_COLOR_RGBA (style->text [GTK_STATE_NORMAL], 0xff);
+
+	g_object_get (G_OBJECT (item), "glyphlist", &gl, NULL);
+	new = gnome_glyphlist_duplicate (gl);
+	for (i = 0; i < new->r_length; i++) {
+		if (new->rules[i].code ==  GGL_COLOR) {
+			new->rules[i].value.ival = color;
+		}
+	}
+	gnome_canvas_item_set (item, "glyphlist", new, NULL);
 }
 
 static int
@@ -249,28 +316,27 @@ gpp_glyphlist (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList * gl
 	GnomeCanvasItem *item;
 	double transform[6], a[6];
 
-
 	/*
 	 * The X and Y positions were computed to be the base
 	 * with the translation already done for the new
 	 * Postscript->Canvas translation
 	 */
-	
 	memcpy (transform, gp_gc_get_ctm (pc->gc), sizeof (transform));
 	art_affine_scale (a, 1.0, -1.0);
 	art_affine_multiply (transform, a, affine);
 
 	group = (GnomeCanvasGroup *) gp_gc_get_data (pp->priv->gc);
-
 	item = gnome_canvas_item_new (group,
-		gnome_canvas_hacktext_get_type (),
-		"x", 0.0,
-		"y", 0.0,
-		"glyphlist", glyphlist,
-		NULL);
+				      gnome_canvas_hacktext_get_type (),
+				      "x", 0.0,
+				      "y", 0.0,
+				      "glyphlist", glyphlist,
+				      NULL);
 
 	gnome_canvas_item_affine_absolute (item, transform);
 
+	if (use_theme)
+		glyphlist_set_style_cb (GTK_WIDGET (item->canvas), item);	
 	return 0;
 }
 
@@ -313,7 +379,7 @@ gpp_finalize (GObject *object)
 	gp_gc_unref (priv->gc);
 
 	if (pp->canvas)
-		gtk_object_unref (GTK_OBJECT (pp->canvas));
+		g_object_unref (G_OBJECT (pp->canvas));
 
 	if (priv->page)
 		gtk_object_destroy (GTK_OBJECT (priv->page));
@@ -337,20 +403,15 @@ class_init (GnomePrintPreviewClass *klass)
 	object_class->finalize = gpp_finalize;
 	
 	pc_class->beginpage = gpp_beginpage;
-	pc_class->showpage = gpp_showpage;
-
-	pc_class->clip = gpp_clip;
-	pc_class->fill = gpp_fill;
-	pc_class->stroke = gpp_stroke;
-
-	pc_class->image = gpp_image;
-
+	pc_class->showpage  = gpp_showpage;
+	pc_class->clip      = gpp_clip;
+	pc_class->fill      = gpp_fill;
+	pc_class->stroke    = gpp_stroke;
+	pc_class->image     = gpp_image;
 	pc_class->glyphlist = gpp_glyphlist;
-	
-	pc_class->gsave = gpp_gsave;
-	pc_class->grestore = gpp_grestore;
-
-	pc_class->close = gpp_close;
+	pc_class->gsave     = gpp_gsave;
+	pc_class->grestore  = gpp_grestore;
+	pc_class->close     = gpp_close;
 }
 
 static void
@@ -367,18 +428,30 @@ clear_val (GtkObject *object, void **val)
 	*val = NULL;
 }
 
-static gint
-gnome_print_preview_construct (GnomePrintContext *ctx)
+void
+gnome_print_preview_use_theme (gboolean theme)
 {
-	GnomePrintPreview *preview;
-
-	preview = GNOME_PRINT_PREVIEW (ctx);
-
-	return GNOME_PRINT_OK;
+	use_theme = theme;
 }
 
-GnomePrintContext *gnome_print_preview_new_full (GnomePrintConfig *config, GnomeCanvas *canvas,
-						 const gdouble *transform, const ArtDRect *region)
+/**
+ * gnome_print_preview_theme_compliance:
+ * @pp: 
+ * @compliance: 
+ * 
+ * This has to go away for GNOME 2.4. Make compliance an argument of
+ * gnome_print_preview_new_full
+ **/
+void
+gnome_print_preview_theme_compliance (GnomePrintPreview *pp,
+				      gboolean compliance)
+{
+	pp->priv->theme_compliance = compliance;
+}
+
+GnomePrintContext *
+gnome_print_preview_new_full (GnomePrintConfig *config, GnomeCanvas *canvas,
+			      const gdouble *transform, const ArtDRect *region)
 {
 	GnomePrintPreview *preview;
 	GnomePrintContext *ctx;
@@ -395,11 +468,12 @@ GnomePrintContext *gnome_print_preview_new_full (GnomePrintConfig *config, Gnome
 	ret = gnome_print_context_construct (ctx, config);
 	if (ret != GNOME_PRINT_OK) {
 		g_object_unref (ctx);
+		g_warning ("Could not construct the GnomePrintPreview context\n");
 		return NULL;
 	}
 	preview = GNOME_PRINT_PREVIEW (ctx);
 
-	gtk_object_ref (GTK_OBJECT (canvas));
+	g_object_ref (G_OBJECT (canvas));
 	preview->canvas = canvas;
 
 	gnome_canvas_set_scroll_region (canvas, region->x0, region->y0, region->x1, region->y1);
@@ -412,10 +486,10 @@ GnomePrintContext *gnome_print_preview_new_full (GnomePrintConfig *config, Gnome
 						     GNOME_TYPE_CANVAS_GROUP,
 						     "x", 0.0, "y", 0.0, NULL);
 
-	gtk_signal_connect (GTK_OBJECT (preview->priv->page), "destroy",
-			    GTK_SIGNAL_FUNC (clear_val), &preview->priv->page);
-	gtk_signal_connect (GTK_OBJECT (preview->priv->root), "destroy",
-			    GTK_SIGNAL_FUNC (clear_val), &preview->priv->root);
+	g_signal_connect (G_OBJECT (preview->priv->page), "destroy",
+			  (GCallback) clear_val, &preview->priv->page);
+	g_signal_connect (G_OBJECT (preview->priv->root), "destroy",
+			  (GCallback) clear_val, &preview->priv->root);
 
 	/* Setup base group */
 	group = GNOME_CANVAS_GROUP (preview->priv->page);
@@ -428,15 +502,14 @@ GnomePrintContext *gnome_print_preview_new_full (GnomePrintConfig *config, Gnome
 
 /**
  * gnome_print_preview_new:
+ * @config:
  * @canvas: Canvas on which we display the print preview
- * @paper_size: A valid name for a paper size
  *
  * Creates a new PrintPreview object that use the @canvas GnomeCanvas 
  * as its rendering backend.
  *
  * Returns: A GnomePrintContext suitable for using with the GNOME print API.
  */
-
 GnomePrintContext *
 gnome_print_preview_new (GnomePrintConfig *config, GnomeCanvas *canvas)
 {
@@ -482,6 +555,7 @@ GType
 gnome_print_preview_get_type (void)
 {
 	static GType preview_type = 0;
+	
 	if (!preview_type) {
 		static const GTypeInfo preview_info = {
 			sizeof (GnomePrintPreviewClass),
